package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.OrganizationRatingMapper;
import com.ims.eval.entity.*;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.util.ExcelUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 年度评级 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-06-20
 */
@Service
public class OrganizationRatingServiceImpl extends ServiceImpl<OrganizationRatingMapper, OrganizationRating> implements IOrganizationRatingService {

	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Resource
	private IOrganizationEvaluationService iOrganizationEvaluationService;

	@Resource
	private IOrganizationRatingService iOrganizationRatingService;

	@Resource
	private IOrganizationEvaluationRuleService iOrganizationEvaluationRuleService;

	@Resource
	private IBinSectionService iBinSectionService;

	@Resource
	private IOrganizationRatingRuleService iOrganizationRatingRuleService;

	@Resource
	private IEvaluationPortalService iEvaluationPortalService;

	@Resource
	private IEvaluateReportService iEvaluateReportService;

	@Resource
	private IEvaluateReportInfoService evaluateReportInfoService;


	/**
	 * 如果重新生成数据则删除之前的评级单位和数据
	 */
	public int clearOldData(String year) {
		LambdaQueryWrapper<OrganizationRating> lqw = new LambdaQueryWrapper<>();
		lqw.eq(OrganizationRating::getYear, year);
		return baseMapper.delete(lqw);
	}


	/**
	 * 导入
	 *
	 * @param file
	 * @return
	 */
	@Override
	public R importExcel(MultipartFile file, String binSection, String year) {
		String binName = null;
		List<BinSection> binSec = CacheContext.bsnList.stream().filter(bin -> binSection.equals(bin.getSectionCode())).collect(Collectors.toList());
		binName = binSec.get(0).getSectionName();

		if (file.isEmpty()) {
			return R.error("上传文件为空！");
		}
		try {
			//获取原始的文件名
			String originalFilename = file.getOriginalFilename();
			//获取文件类型
			String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
			//默认从第一行开始读取
			int startRows = 1;
			//获取输入流
			InputStream is = file.getInputStream();
			List<OrganizationRating> orgList = new ArrayList<>();
			//Excel导入导出的单元类
			List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
			Page<OrganizationRating> page = new Page<>(1, 10000);
			IPage<OrganizationRating> organizationRatingIPage = baseMapper.selectList(page, null, null, binSection, null, null, null, null, null, null, null, null, null, null, null, null);
			List<OrganizationRating> orgRatings = organizationRatingIPage.getRecords().stream().filter(org -> binSection.equals(org.getBinSection()) && year.equals(org.getYear())).collect(Collectors.toList());
			//遍历Excel表每一行的数据
			for (String[] str : strings) {
				if (!str[1].equals(binName)) {
					return R.error("请导入" + binName + "的数据");
				}
				if (!str[2].equals(orgRatings.get(0).getYear())) {
					return R.error("请导入" + year + "年的数据");
				}
				for (OrganizationRating or : orgRatings) {
					if (str[0].equals(or.getOrganizationShortName()) && str[2].equals(or.getYear())) {
						or.setSafetyEnvProtection(str[3]);
						or.setVeto(str[4]);
						or.setNetProfit(str[5]);
						or.setCompositeScore(str[6]);
						or.setScjyRate(str[7]);
						or.setAnnualTarget(str[8]);
						or.setBonusPoint(str[9]);
						or.setQqRate(str[10]);
						or.setCompositeScoreRaking(str[11]);
						or.setProjectLevel(str[12]);
						or.setJjRate(str[13]);
						or.setScore(str[14]);
						or.setYearRate(str[15]);
						orgList.add(or);
					}
				}
			}
			boolean b = super.saveOrUpdateBatch(orgList);
			savePortal(orgList);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("导入失败！");
			}
		} catch (Exception e) {
			return R.customError(e.getMessage()).data("失败！");
		}

	}


	/**
	 * 导出
	 */
	@Override
	public R exportExcel(HttpServletResponse response, List<OrganizationRating> ls, String binSection, String year) throws IOException {
		// 设置响应头信息，以附件形式下载
		response.setHeader("Content-Disposition", "attachment; filename=organization_rating.xlsx");
		response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		// 创建新的Excel工作簿
		Workbook workbook = new XSSFWorkbook();

		// 创建新的工作表
		Sheet sheet = workbook.createSheet("Sheet1");

		// 创建表头
		Row headerRow = sheet.createRow(0);
		List<String> headers = Arrays.asList("考评单位", "业务板块", "考评年度", "年度安全环保评级", "否决事项:无0,有1", "净利润", "经营业绩考核综合得分%", "生产经营评级", "年度项目进度目标完成率",
			"先进性指标考核得分有无加分", "前期评级", "基建工程综合得分排名", "基建工程项目等级", "基建评级", "综合得分", "年度评级");

		for (int i = 0; i < headers.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellValue(headers.get(i));
		}

		//行数据
//		Page<OrganizationRating> page = new Page<>(1, 100000);
//		IPage<OrganizationRating> orgRating = baseMapper.selectList(page, null, null, binSection, year, null, null,
//			null, null, null, null, null, null, null, null, null);

		// 填充数据到工作表
		int rowIndex = 1; // 从第二行开始填充数据（第一行是表头）
		for (OrganizationRating or : ls) {
			Row row = sheet.createRow(rowIndex++);

			// 设置数据
			Cell cell0 = row.createCell(0);
			cell0.setCellValue(or.getOrganizationShortName());
			Cell cell1 = row.createCell(1);
			cell1.setCellValue(or.getBinSectionName());
			Cell cell2 = row.createCell(2);
			cell2.setCellValue(or.getYear());
			Cell cell3 = row.createCell(3);
			cell3.setCellValue(or.getSafetyEnvProtection());
			Cell cell4 = row.createCell(4);
			cell4.setCellValue(or.getVeto());
			Cell cell5 = row.createCell(5);
			cell5.setCellValue(or.getNetProfit());
			Cell cell6 = row.createCell(6);
			cell6.setCellValue(or.getCompositeScore());
			Cell cell7 = row.createCell(7);
			cell7.setCellValue(or.getScjyRate());
			Cell cell8 = row.createCell(8);
			cell8.setCellValue(or.getAnnualTarget());
			Cell cell9 = row.createCell(9);
			cell9.setCellValue(or.getBonusPoint());
			Cell cell10 = row.createCell(10);
			cell10.setCellValue(or.getQqRate());
			Cell cell11 = row.createCell(11);
			cell11.setCellValue(or.getCompositeScoreRaking());
			Cell cell12 = row.createCell(12);
			cell12.setCellValue(or.getProjectLevel());
			Cell cell13 = row.createCell(13);
			cell13.setCellValue(or.getJjRate());
			Cell cell14 = row.createCell(14);
			cell14.setCellValue(or.getScore());
			Cell cell15 = row.createCell(15);
			cell15.setCellValue(or.getYearRate());
		}
		// 自动调整列宽
		for (int i = 0; i < headers.size() - 1; i++) {
			sheet.autoSizeColumn(i);
		}

		// 将工作簿写入到输出流
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
			workbook.write(outputStream);
			response.getOutputStream().write(outputStream.toByteArray());
			response.getOutputStream().flush();
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			// 关闭工作簿
			workbook.close();
		}
		return R.ok();
	}


	/**
	 * 单位考评评级
	 */
	@Override
	public R grade(List<OrganizationRating> orgRating) {
		XSSFWorkbook xwb = new XSSFWorkbook();
		Sheet st = xwb.createSheet();
		List<OrganizationRatingRule> rules = iOrganizationRatingRuleService.list(null);
		if (rules == null || rules.isEmpty()) {
			return R.error("没有评级公式，请配置评级公式");
		}
		List<OrganizationRatingRule> rulesYear = rules.stream().filter(rul -> orgRating.get(0).getYear().equals(rul.getYear())).collect(Collectors.toList());

		Map<String, List<OrganizationRating>> collect = orgRating.stream().collect(Collectors.groupingBy(OrganizationRating::getBinSection));
		Map<String, List<OrganizationRating>> col = new HashMap<>();
		collect.forEach((key, value) -> CacheContext.bsnList.stream()
			.filter(b -> key.equals(b.getSectionCode()))
			.findFirst()
			.ifPresent(b -> col.put(b.getId(), value)));

		R r1 = calcSCJYrating(rulesYear, col, st, xwb);
		R r2 = calcQQXMrating(rulesYear, col, st, xwb);
		R r3 = calcJJXMrating(rulesYear, col, st, xwb);
		R r4 = calcNDrating(rulesYear, orgRating, st, xwb);
		if (r1.getSuccess().equals(false) && r2.getSuccess().equals(false) && r3.getSuccess().equals(false)) {
			return R.error("评级失败");
		}
		Map<String, OrganizationRating> map = orgRating.stream().collect(Collectors.toMap(
			OrganizationRating::getId,
			Function.identity(),
			(existing, replacement) -> existing,
			LinkedHashMap::new));
		List<OrganizationRating> organizationRatings = baseMapper.selectBatchIds(map.keySet());
		savePortal(organizationRatings);
//		return organizationRatings;
		return R.ok().data(organizationRatings);
	}


	/**
	 * 保存数据到首页
	 */
	private void savePortal(List<OrganizationRating> organizationRatings) {
		List<EvaluationPortal> lep = new ArrayList<>();
		for (OrganizationRating or : organizationRatings) {
			if (or.getYearRate() != null && or.getScore() != null) {
				EvaluationPortal ep = new EvaluationPortal();
				ep.setBinSection(or.getBinSection());
				ep.setCompany(or.getOrganizationShortName());
				ep.setGrade(or.getYearRate());
				ep.setScore(Double.parseDouble(or.getScore()));
				ep.setBaseScore(150.00);
				ep.setYear(or.getYear());
				lep.add(ep);
			}
		}
//		boolean b = iEvaluationPortalService.saveBatch(lep);
		List<EvaluationPortal> leUpdate = new ArrayList<>();
		List<EvaluationPortal> leSave = new ArrayList<>();
		for (EvaluationPortal evaluationPortal : lep) {
			LambdaQueryWrapper<EvaluationPortal> lqw = new LambdaQueryWrapper<>();
			lqw.eq(EvaluationPortal::getCompany, evaluationPortal.getCompany()).eq(EvaluationPortal::getYear, evaluationPortal.getYear())
				.isNull(EvaluationPortal::getSeason).eq(EvaluationPortal::getBinSection, evaluationPortal.getBinSection());
			List<EvaluationPortal> list = iEvaluationPortalService.list(lqw);
			if (null == list || list.isEmpty()) {
				leSave.add(evaluationPortal);
			} else {
				evaluationPortal.setId(list.get(0).getId());
				leUpdate.add(evaluationPortal);
			}
		}
		boolean save = iEvaluationPortalService.saveBatch(leSave);
		boolean update = iEvaluationPortalService.updateBatchById(leUpdate);
	}


	/**
	 * 年度评级
	 */
	private R calcNDrating(List<OrganizationRatingRule> list, List<OrganizationRating> orgRating, Sheet st, XSSFWorkbook xwb) {
		List<String> ls = new ArrayList<>();
		for (OrganizationRating or : orgRating) {
			ls.add(or.getId());
		}
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "年度评级".equals(orr.getBinStage())).collect(Collectors.toList());
		List<OrganizationRating> org = iOrganizationRatingService.listByIds(ls);
		Map<String, List<OrganizationRating>> col = org.stream().collect(Collectors.groupingBy(OrganizationRating::getBinSection));

		Map<String, List<OrganizationRating>> col2 = new HashMap<>();
		col.forEach((key, value) -> CacheContext.bsnList.stream()
			.filter(b -> key.equals(b.getSectionCode()))
			.findFirst()
			.ifPresent(b -> col2.put(b.getId(), value)));

		for (Map.Entry<String, List<OrganizationRating>> orgMap : col2.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			if (collect.isEmpty()) {
				continue;
			}
			String formula = collect.get(0).getExpression();

			for (OrganizationRating or : orgMap.getValue()) {
				if (or.getScjyRate() == null || or.getScjyRate().isEmpty() || or.getQqRate() == null || or.getQqRate().isEmpty()) {
					continue;
				}
				String replace = formula;
				Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");//编译成模式
				Matcher matcher = pattern.matcher(formula);
				Map<String, String> placeholders = new HashMap<>();
				String key = null;
				while (matcher.find()) {
					String content = matcher.group(1);
					if ("scjy_rate".equals(content)) {
						if (or.getScjyRate() == null) {
							return R.error();
						}
						key = or.getScjyRate();
					} else if ("qq_rate".equals(content)) {
						if (or.getQqRate() == null) {
							return R.error();
						}
						key = or.getQqRate();
					}
					if (key != null) {
						replace = replace.replace("[" + content + "]", key);
					}
					placeholders.put(key, content);
				}
				String form = replace;
				Row rr = st.createRow(0);
				Cell cc = rr.createCell(0);
				// 这里要设置为公式类型
				try {
					cc.setCellFormula(form);
				} catch (Exception e) {
					return R.error("公式错误:" + form);
				}
				FormulaEvaluator fe = new XSSFFormulaEvaluator(xwb);
				CellValue cv = fe.evaluate(cc);
				UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
				uw.eq("id", or.getId()).set("year_rate", cv.formatAsString());
				iOrganizationRatingService.update(uw);
			}
		}
		return R.ok();
	}


	/*private CellValue setFormula(Sheet st, XSSFWorkbook xwb, String form) {
		Row rr = st.createRow(0);
		Cell cc = rr.createCell(0);
		// 这里要设置为公式类型
		try {
			cc.setCellFormula(form);
		} catch (Exception e) {
			R.error("公式错误");
		}
		FormulaEvaluator fe = new XSSFFormulaEvaluator(xwb);
		return fe.evaluate(cc);
	}*/


	/**
	 * 基建评级
	 */
	private R calcJJXMrating(List<OrganizationRatingRule> list, Map<String, List<OrganizationRating>> orgRating, Sheet st, XSSFWorkbook xwb) {
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "基建评级".equals(orr.getBinStage())).collect(Collectors.toList());
		for (Map.Entry<String, List<OrganizationRating>> orgMap : orgRating.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			if (collect.isEmpty()) {
				continue;
			}
			String formula = collect.get(0).getExpression();
			for (OrganizationRating or : orgMap.getValue()) {
				if (or.getCompositeScoreRaking() == null || or.getCompositeScoreRaking().isEmpty() || or.getProjectLevel() == null || or.getProjectLevel().isEmpty()) {
					continue;
				}
				String replace = formula;
				Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");//编译成模式
				Matcher matcher = pattern.matcher(formula);
				Map<String, String> placeholders = new HashMap<>();
				String key = null;
				while (matcher.find()) {
					String content = matcher.group(1);
					if ("ZHDEPM".equals(content)) {
						key = or.getCompositeScoreRaking();
					} else if ("JJDJ".equals(content)) {
						key = or.getProjectLevel();
					}
					if (key != null) {
						replace = replace.replace("[" + content + "]", key);
					}
					placeholders.put(key, content);
				}
				String form = replace;
				Row rr = st.createRow(0);
				Cell cc = rr.createCell(0);
				// 这里要设置为公式类型
				try {
					cc.setCellFormula(form);
				} catch (Exception e) {
					return R.error("公式错误:" + form);
				}
				FormulaEvaluator fe = new XSSFFormulaEvaluator(xwb);
				CellValue cv = fe.evaluate(cc);
				UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
				uw.eq("id", or.getId()).set("jj_rate", cv.formatAsString()).set("composite_score_raking", or.getCompositeScoreRaking())
					.set("project_level", or.getProjectLevel());
				iOrganizationRatingService.update(uw);
			}
		}
		return R.ok();
	}

	/**
	 * 生产经营评级
	 */
	private R calcSCJYrating(List<OrganizationRatingRule> list, Map<String, List<OrganizationRating>> orgRating, Sheet st, XSSFWorkbook xwb) {
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "生产经营评级".equals(orr.getBinStage())).collect(Collectors.toList());
		for (Map.Entry<String, List<OrganizationRating>> orgMap : orgRating.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			for (OrganizationRatingRule ru : rules) {
				System.out.println("key+" + orgMap.getKey() + "---" + ru.getBinSection());
			}
			if (collect.isEmpty()) {
				continue;
			}
			String formula = collect.get(0).getExpression();
			for (OrganizationRating or : orgMap.getValue()) {
				if (or.getSafetyEnvProtection() == null || or.getSafetyEnvProtection().isEmpty() || or.getNetProfit() == null || or.getNetProfit().isEmpty() || or.getCompositeScore() == null || or.getCompositeScore().isEmpty() || or.getVeto() == null || or.getVeto().isEmpty()) {
					continue;
				}
				String replace = formula;
				Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");//编译成模式
				Matcher matcher = pattern.matcher(formula);
				Map<String, String> placeholders = new HashMap<>();
				String key = null;
				while (matcher.find()) {
					String content = matcher.group(1);
					if ("AQHB".equals(content)) {
						key = or.getSafetyEnvProtection();
					} else if ("JLR".equals(content)) {
						key = or.getNetProfit();
					} else if ("ZHDF".equals(content)) {
						key = or.getCompositeScore();
					} else if ("FJSX".equals(content)) {
						key = or.getVeto();
					}
					if (key != null) {
						replace = replace.replace("[" + content + "]", key);
					}
					placeholders.put(key, content);
				}
				String form = replace;
				Row rr = st.createRow(0);
				Cell cc = rr.createCell(0);
				// 这里要设置为公式类型
				try {
					cc.setCellFormula(form);
				} catch (Exception e) {
					return R.error("公式错误:" + form);
				}
				FormulaEvaluator fe = new XSSFFormulaEvaluator(xwb);
				CellValue cv = fe.evaluate(cc);
				UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
				uw.eq("id", or.getId()).set("scjy_rate", cv.formatAsString()).set("safety_env_protection", or.getSafetyEnvProtection())
					.set("veto", or.getVeto()).set("net_profit", or.getNetProfit()).set("composite_score", or.getCompositeScore());
				iOrganizationRatingService.update(uw);
			}
		}
		return R.ok();
	}

	/**
	 * 前期评级
	 */
	private R calcQQXMrating(List<OrganizationRatingRule> list, Map<String, List<OrganizationRating>> orgRating, Sheet st, XSSFWorkbook xwb) {
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "前期评级".equals(orr.getBinStage())).collect(Collectors.toList());
		for (Map.Entry<String, List<OrganizationRating>> orgMap : orgRating.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			if (collect.isEmpty()) {
				continue;
			}
			String formula = collect.get(0).getExpression();
			for (OrganizationRating or : orgMap.getValue()) {
				if (or.getAnnualTarget() == null || or.getAnnualTarget().isEmpty() || or.getBonusPoint() == null || or.getBonusPoint().isEmpty()) {
					continue;
				}
				String replace = formula;
				Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");
				Matcher matcher = pattern.matcher(formula);
				Map<String, String> placeholders = new HashMap<>();
				String key = null;
				while (matcher.find()) {
					String content = matcher.group(1);
					if ("NDMB".equals(content)) {
						key = or.getAnnualTarget();
					} else if ("XJX".equals(content)) {
						key = or.getBonusPoint();
					}
					if (key != null) {
						replace = replace.replace("[" + content + "]", key);
					}
					placeholders.put(key, content);
				}
				String form = replace;
				Row rr = st.createRow(0);
				Cell cc = rr.createCell(0);
				// 这里要设置为公式类型
				try {
					cc.setCellFormula(form);
				} catch (Exception e) {
					return R.error("公式错误:" + form);
				}
				FormulaEvaluator fe = new XSSFFormulaEvaluator(xwb);
				CellValue cv = fe.evaluate(cc);
				UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
				uw.eq("id", or.getId()).set("qq_rate", cv.formatAsString()).set("annual_target", or.getAnnualTarget()).set("bonus_point", or.getBonusPoint());
				iOrganizationRatingService.update(uw);
			}
		}
		return R.ok();
	}


	@Override
	public IPage<OrganizationRating> list(Integer pageNum, Integer pageSize, String id, String
		organizationYearRatingId, String binSection, String year, String organizationShortName, String organizationId, String safetyEnvProtection,
										  String veto, String netProfit, String compositeScore, String annualTarget, String bonusPoint, String compositeScoreRaking, String projectLevel, String score) {
		List<BinSection> bsnList = CacheContext.bsnList;
		List<BinSection> bsnls = bsnList.stream().filter(s -> null != s.getMemo1() && s.getMemo1().equals("DWPJ")).collect(Collectors.toList());
		Map<String, List<BinSection>> bsnMap = bsnls.stream().collect(Collectors.groupingBy(BinSection::getId));
		String[] s = organizationYearRatingId.split("_");

		Page<OrganizationRating> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}

		IPage<OrganizationRating> list = baseMapper.selectList(page, id, organizationYearRatingId, binSection, year, organizationShortName,
			organizationId, safetyEnvProtection, veto, netProfit, compositeScore, annualTarget, bonusPoint, compositeScoreRaking, projectLevel, score);

		if (list.getRecords() == null || list.getRecords().isEmpty()) {
			IPage<OrganizationRating> ll = baseMapper.selectList(new Page<>(1, 1000000), null, null, null, null, null,
				null, null, null, null, null, null, null, null, null, null);

			if (ll.getRecords() == null || ll.getRecords().isEmpty()) {
				getRatingInfo(organizationYearRatingId, year, s[0], bsnMap);
				return baseMapper.selectList(page, id, organizationYearRatingId, binSection, year, organizationShortName, organizationId,
					safetyEnvProtection, veto, netProfit, compositeScore, annualTarget, bonusPoint, compositeScoreRaking, projectLevel, score);
			}
			Map<String, List<OrganizationRating>> groupByYear = ll.getRecords().stream().collect(Collectors.groupingBy(OrganizationRating::getYear));
			if (groupByYear.containsKey(year)) {
				return list;
			}
			getRatingInfo(organizationYearRatingId, year, s[0], bsnMap);
			return baseMapper.selectList(page, id, organizationYearRatingId, binSection, year, organizationShortName, organizationId,
				safetyEnvProtection, veto, netProfit, compositeScore, annualTarget, bonusPoint, compositeScoreRaking, projectLevel, score);
		}
		return list;
	}


	@Override
	public R binSection(String organizationYearRatingId) {
		List<BinSection> bsnList = CacheContext.bsnList;

		Map<String, List<BinSection>> bsnMap = bsnList.stream().collect(Collectors.groupingBy(BinSection::getId));
		String[] s = organizationYearRatingId.split("_");

		List<EvaluateReport> evaReport = iEvaluateReportService.list(s[0], "");
		if (null == evaReport || evaReport.isEmpty()) {
			return R.ok();
		}
		Set<BinSection> lb = new HashSet<>();
		for (EvaluateReport evaluateReport : evaReport) {
			LambdaQueryWrapper<EvaluateReportInfo> lqw = new LambdaQueryWrapper<>();
			lqw.eq(EvaluateReportInfo::getEvaluateReportId, evaluateReport.getId());
			List<EvaluateReportInfo> evaReportInfos = evaluateReportInfoService.list(lqw);
			if (null != evaReportInfos && !(evaReportInfos.isEmpty())) {
				lb.add(bsnMap.get(evaluateReport.getBinSection()).get(0));
			}
		}
		List<BinSection> lbs = new ArrayList<>(lb);
		for (BinSection binSection : lbs) {
			if ("HD".equals(binSection.getSectionCode())) {
				binSection.setMemo4("1");
			}else if ("SD".equals(binSection.getSectionCode())) {
				binSection.setMemo4("2");
			}else if ("XNYFG".equals(binSection.getSectionCode())) {
				binSection.setMemo4("3");
			}else if ("MDYTH".equals(binSection.getSectionCode())) {
				binSection.setMemo4("4");
			}else if ("GJYW".equals(binSection.getSectionCode())) {
				binSection.setMemo4("5");
			}else if ("ZHZC".equals(binSection.getSectionCode())) {
				binSection.setMemo4("6");
			}else if ("TJGS".equals(binSection.getSectionCode())) {
				binSection.setMemo4("7");
			}else if ("GCGS".equals(binSection.getSectionCode())) {
				binSection.setMemo4("8");
			}else if ("QQ".equals(binSection.getSectionCode())) {
				binSection.setMemo4("9");
			}else if ("JJ".equals(binSection.getSectionCode())) {
				binSection.setMemo4("10");
			}
		}
		List<BinSection> col = lbs.stream().sorted(Comparator.comparing(BinSection::getMemo4)).collect(Collectors.toList());
		return R.ok().data(col);
	}


	private void getRatingInfo(String organizationYearRatingId, String year, String s, Map<String, List<BinSection>> bsnMap) {
		List<EvaluateReport> evaReport = iEvaluateReportService.list(s, "");
		if (null == evaReport || evaReport.isEmpty()) {
			throw new RuntimeException("考评报告未生成，请先生成考评报告");
		}
//		Set<BinSection> lb = new HashSet<>();
		List<OrganizationRating> lors = new ArrayList<>();
		for (EvaluateReport evaluateReport : evaReport) {
			LambdaQueryWrapper<EvaluateReportInfo> lqw = new LambdaQueryWrapper<>();
			lqw.eq(EvaluateReportInfo::getEvaluateReportId, evaluateReport.getId());
			List<EvaluateReportInfo> evaReportInfos = evaluateReportInfoService.list(lqw);
			if (null == evaReportInfos || evaReportInfos.isEmpty()) {
				continue;
			}
			for (EvaluateReportInfo evaReportInfo : evaReportInfos) {
				OrganizationRating or = new OrganizationRating();
				or.setOrganizationYearRatingId(organizationYearRatingId);
				or.setOrganizationShortName(evaReportInfo.getOrganizationName());
				or.setOrganizationId(evaReportInfo.getOrganizationId());
				or.setYear(year);
				or.setBinSection(bsnMap.get(evaluateReport.getBinSection()).get(0).getSectionCode());
				or.setScore(evaReportInfo.getScore() != 0.0 ? String.valueOf(evaReportInfo.getScore()) : null);
				lors.add(or);
//				lb.add(bsnMap.get(evaluateReport.getBinSection()).get(0));
			}
		}
		iOrganizationRatingService.saveBatch(lors);
	}


	/**
	 * 根据年度考评公司写年度等级表中的id
	 */
	private void getOrgRatingInfo(String organizationYearRatingId) {
		LambdaQueryWrapper<OrganizationEvaluation> lqw = new LambdaQueryWrapper<>();
		lqw.eq(OrganizationEvaluation::getCheckCycle, "NDKP");
		List<OrganizationEvaluation> ll = iOrganizationEvaluationService.list(lqw);
		if (ll.isEmpty()) {
			return;
		}
		OrganizationEvaluation yearMax = null;
		if (ll.size() > 1) {
			yearMax = ll.stream().filter(ye -> ye.getYear().matches("\\d+")).max(Comparator.comparing(ye -> Integer.parseInt(ye.getYear()))).orElse(null);
		} else {
			yearMax = ll.get(0);
		}
		if (yearMax == null) {
			return;
		}
		List<OrganizationRating> lors = new ArrayList<>();
		String ruls = yearMax.getOrganizationEvaluationRuleId();
		String[] ru = ruls.split(",");
		List<OrganizationEvaluationRule> oers = iOrganizationEvaluationRuleService.listByIds(Arrays.asList(ru));
		List<BinSection> binList = iBinSectionService.list();
		for (OrganizationEvaluationRule oer : oers) {
			if (!oer.getBinSection().contains(",")) {
				OrganizationRating or = new OrganizationRating();
				or.setYear(yearMax.getYear());
				or.setOrganizationShortName(oer.getOrganizationShortName());
				or.setOrganizationId(oer.getOrganizationId());
				or.setOrganizationYearRatingId(organizationYearRatingId);
				for (BinSection bl : binList) {
					if (bl.getId().equals(oer.getBinSection())) {
						or.setBinSection(bl.getSectionCode());
						break;
					}
				}
				lors.add(or);
				continue;
			}
			String[] split = oer.getBinSection().split(",");
			Set<String> splitSet = new LinkedHashSet<>(Arrays.asList(split));
			for (String s : splitSet) {
				OrganizationRating or = new OrganizationRating();
				or.setYear(yearMax.getYear());
				or.setOrganizationShortName(oer.getOrganizationShortName());
				or.setOrganizationId(oer.getOrganizationId());
				or.setOrganizationYearRatingId(organizationYearRatingId);
				for (BinSection bl : binList) {
					if (bl.getId().equals(s)) {
						or.setBinSection(bl.getSectionCode());
						break;
					}
				}
				lors.add(or);
			}
		}
		iOrganizationRatingService.saveBatch(lors);
	}


	@Override
	public boolean generateRating(String organizationYearRatingId, String year) {
		List<OrganizationRating> ratings = getListSating(organizationYearRatingId, year, "");
		List<String> collect = ratings.stream().map(OrganizationRating::getOrganizationId).collect(Collectors.toList());
		List<OrganizationEvaluationRule> evaluationRules = organizationEvaluationRuleService.listByIsCheck("DWKP", "NDKP", year, true, false);

		List<OrganizationRating> addRatings = new ArrayList<>();
		for (OrganizationEvaluationRule rule : evaluationRules) {
			if (null != collect && collect.size() > 0 && collect.contains(rule.getOrganizationId())) {
				continue;
			}
			OrganizationRating rating = new OrganizationRating();
			rating.setOrganizationYearRatingId(organizationYearRatingId);
			rating.setOrganizationShortName(rule.getOrganizationShortName());
			rating.setOrganizationId(rule.getOrganizationId());
			rating.setYear(year);
			rating.setYearRate("A");
			rating.setBinSection(rule.getBinSection().split(",")[0]);
			addRatings.add(rating);
		}

		if (addRatings.size() <= 0) {
			return true;
		}
		boolean b = this.saveOrUpdateBatch(addRatings);
		return b;
	}

	@Override
	public List<OrganizationRating> getListSating(String organizationYearRatingId, String year, String
		organizationId) {

		List<OrganizationRating> list = baseMapper.selectListSating(organizationYearRatingId, year, organizationId);
		return list;
	}
}
